from logging import getLogger

from qbittorrentapi import Version
from qbittorrentapi.decorators import login_required
from qbittorrentapi.definitions import APINames
from qbittorrentapi.definitions import ClientCache
from qbittorrentapi.exceptions import HTTP403Error
from qbittorrentapi.exceptions import LoginFailed
from qbittorrentapi.exceptions import UnsupportedQbittorrentVersion
from qbittorrentapi.request import Request

logger = getLogger(__name__)


class Authorization(ClientCache):
    """
    Allows interaction with the ``Authorization`` API endpoints.

    :Usage:
        >>> from qbittorrentapi import Client
        >>> client = Client(host='localhost:8080', username='admin', password='adminadmin')
        >>> is_logged_in = client.auth.is_logged_in
        >>> client.auth.log_in(username='admin', password='adminadmin')
        >>> client.auth.log_out()
    """

    @property
    def is_logged_in(self):
        """Implements :meth:`~AuthAPIMixIn.is_logged_in`"""
        return self._client.is_logged_in

    def log_in(self, username=None, password=None, **kwargs):
        """Implements :meth:`~AuthAPIMixIn.auth_log_in`"""
        return self._client.auth_log_in(username=username, password=password, **kwargs)

    def log_out(self, **kwargs):
        """Implements :meth:`~AuthAPIMixIn.auth_log_out`"""
        return self._client.auth_log_out(**kwargs)


class AuthAPIMixIn(Request):
    """
    Implementation of all ``Authorization`` API methods.

    :Usage:
        >>> from qbittorrentapi import Client
        >>> client = Client(host='localhost:8080', username='admin', password='adminadmin')
        >>> _ = client.is_logged_in
        >>> client.auth_log_in(username='admin', password='adminadmin')
        >>> client.auth_log_out()
    """

    @property
    def auth(self):
        """
        Allows for transparent interaction with Authorization endpoints.

        :return: Auth object
        """
        if self._authorization is None:
            self._authorization = Authorization(client=self)
        return self._authorization

    authorization = auth

    @property
    def is_logged_in(self):
        """
        Returns True if low-overhead API call succeeds; False otherwise.

        There isn't a reliable way to know if an existing session is still valid
        without attempting to use it. qBittorrent invalidates cookies when they expire.

        :returns: True/False if current auth cookie is accepted by qBittorrent.
        """
        try:
            self._post(_name=APINames.Application, _method="version")
        except HTTP403Error:
            return False
        else:
            return True

    def auth_log_in(self, username=None, password=None, **kwargs):
        """
        Log in to qBittorrent host.

        :raises LoginFailed: if credentials failed to log in
        :raises Forbidden403Error: if user is banned...or not logged in

        :param username: username for qBittorrent client
        :param password: password for qBittorrent client
        :return: None
        """
        if username:
            self.username = username
            self._password = password or ""

        self._initialize_context()
        creds = {"username": self.username, "password": self._password}
        auth_response = self._post(
            _name=APINames.Authorization, _method="login", data=creds, **kwargs
        )

        if auth_response.text != "Ok.":
            logger.debug("Login failed")
            raise LoginFailed()
        logger.debug("Login successful")

        # check if the connected qBittorrent is fully supported by this Client yet
        if self._RAISE_UNSUPPORTEDVERSIONERROR:
            app_version = self.app_version()
            api_version = self.app_web_api_version()
            if not (
                Version.is_api_version_supported(api_version)
                and Version.is_app_version_supported(app_version)
            ):
                raise UnsupportedQbittorrentVersion(
                    "This version of qBittorrent is not fully supported => App Version: %s API Version: %s"
                    % (app_version, api_version)
                )

    @property
    def _SID(self):
        """
        Authorization session cookie from qBittorrent using default cookie name
        `SID`. Backwards compatible for :meth:`~AuthAPIMixIn._session_cookie`.

        :return: Auth cookie value from qBittorrent or None if one isn't already acquired
        """
        return self._session_cookie()

    def _session_cookie(self, cookie_name="SID"):
        """
        Authorization session cookie from qBittorrent.

        :param cookie_name: Name of the authorization cookie; configurable after v4.5.0.
        :return: Auth cookie value from qBittorrent or None if one isn't already acquired
        """
        if self._http_session:
            return self._http_session.cookies.get(cookie_name, None)
        return None

    @login_required
    def auth_log_out(self, **kwargs):
        """End session with qBittorrent."""
        self._post(_name=APINames.Authorization, _method="logout", **kwargs)



from abc import ABCMeta
from typing import Any
from typing import Dict
from typing import Mapping
from typing import MutableMapping
from typing import Text
from typing import TypeVar

K = TypeVar("K")
KOther = TypeVar("KOther")
V = TypeVar("V")
VOther = TypeVar("VOther")
KwargsT = Any

def merge(
    left: Mapping[K, V],
    right: Mapping[KOther, VOther],
) -> Dict[K | KOther, V | VOther]: ...

class Attr(Mapping[K, V], metaclass=ABCMeta):
    def __call__(self, key: K) -> V: ...
    def __getattr__(self, key: Text) -> V: ...
    def __add__(
        self,
        other: Mapping[KOther, VOther],
    ) -> Attr[K | KOther, V | VOther]: ...
    def __radd__(
        self,
        other: Mapping[KOther, VOther],
    ) -> Attr[K | KOther, V | VOther]: ...

class MutableAttr(Attr[K, V], MutableMapping[K, V], metaclass=ABCMeta):
    def __setattr__(self, key: Text, value: V) -> None: ...
    def __delattr__(self, key: Text, force: bool = ...) -> None: ...

class AttrDict(Dict[K, V], MutableAttr[K, V]):
    def __init__(self, *args: Any, **kwargs: KwargsT) -> None: ...



from typing import Dict
from typing import Optional
from typing import Set
from typing import Text

from pkg_resources.extern.packaging.version import Version as _Version  # type: ignore

MOST_RECENT_SUPPORTED_APP_VERSION: Text
MOST_RECENT_SUPPORTED_API_VERSION: Text
APP_VERSION_2_API_VERSION_MAP: Dict[Text, Text]

def v(version: Text) -> _Version: ...  # type: ignore

class Version:
    _supported_app_versions: Optional[Set[str]] = None
    _supported_api_versions: Optional[Set[str]] = None
    @classmethod
    def supported_app_versions(cls) -> Set[str]: ...
    @classmethod
    def supported_api_versions(cls) -> Set[str]: ...
    @classmethod
    def is_app_version_supported(cls, app_version: Text) -> bool: ...
    @classmethod
    def is_api_version_supported(cls, api_version: Text) -> bool: ...
    @classmethod
    def latest_supported_app_version(cls) -> str: ...
    @classmethod
    def latest_supported_api_version(cls) -> str: ...



from qbittorrentapi import Client


def test_is_logged_in():
    client = Client(
        RAISE_NOTIMPLEMENTEDERROR_FOR_UNIMPLEMENTED_API_ENDPOINTS=True,
        VERIFY_WEBUI_CERTIFICATE=False,
    )
    assert client.is_logged_in is False

    client.auth_log_in()
    assert client.is_logged_in is True

    client.auth_log_out()
    assert client.is_logged_in is False


def test_session_cookie(app_version):
    client = Client(
        RAISE_NOTIMPLEMENTEDERROR_FOR_UNIMPLEMENTED_API_ENDPOINTS=True,
        VERIFY_WEBUI_CERTIFICATE=False,
    )
    assert client._session_cookie() is None

    # make the client perform a login
    assert client.app.version == app_version

    # should test other cookie names but it's difficult to change
    curr_sess_cookie = client._http_session.cookies["SID"]
    assert client._SID == curr_sess_cookie
    assert client._session_cookie() == curr_sess_cookie



from typing import Iterable
from typing import Optional
from typing import Text

from qbittorrentapi._types import JsonValueT
from qbittorrentapi._types import KwargsT
from qbittorrentapi.app import AppAPIMixIn
from qbittorrentapi.definitions import ClientCache
from qbittorrentapi.definitions import Dictionary

# mypy crashes when this is imported from _types...
JsonDictionaryT = Dictionary[Text, JsonValueT]

class TransferInfoDictionary(JsonDictionaryT): ...

class Transfer(ClientCache):
    @property
    def info(self) -> TransferInfoDictionary: ...
    @property
    def speed_limits_mode(self) -> Text: ...
    @speed_limits_mode.setter
    def speed_limits_mode(self, v: Text | int) -> None: ...
    @property
    def speedLimitsMode(self) -> Text: ...
    @speedLimitsMode.setter
    def speedLimitsMode(self, v: Text | int) -> None: ...
    def set_speed_limits_mode(
        self,
        intended_state: Optional[bool] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    setSpeedLimitsMode = set_speed_limits_mode
    def toggleSpeedLimitsMode(
        self,
        intended_state: Optional[bool] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def toggle_speed_limits_mode(
        self,
        intended_state: Optional[bool] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    @property
    def download_limit(self) -> int: ...
    @download_limit.setter
    def download_limit(self, v: Text | int) -> None: ...
    @property
    def downloadLimit(self) -> int: ...
    @downloadLimit.setter
    def downloadLimit(self, v: Text | int) -> None: ...
    @property
    def upload_limit(self) -> int: ...
    @upload_limit.setter
    def upload_limit(self, v: Text | int) -> None: ...
    @property
    def uploadLimit(self) -> int: ...
    @uploadLimit.setter
    def uploadLimit(self, v: Text | int) -> None: ...
    def set_download_limit(
        self,
        limit: Optional[Text | int] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def setDownloadLimit(
        self,
        limit: Optional[Text | int] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def set_upload_limit(
        self,
        limit: Optional[Text | int] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def setUploadLimit(
        self,
        limit: Optional[Text | int] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def ban_peers(
        self,
        peers: Optional[Text | Iterable[Text]] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def banPeers(
        self,
        peers: Optional[Text | Iterable[Text]] = None,
        **kwargs: KwargsT,
    ) -> None: ...

class TransferAPIMixIn(AppAPIMixIn):
    @property
    def transfer(self) -> Transfer: ...
    def transfer_info(self, **kwargs: KwargsT) -> TransferInfoDictionary: ...
    def transfer_speed_limits_mode(self, **kwargs: KwargsT) -> str: ...
    def transfer_speedLimitsMode(self, **kwargs: KwargsT) -> str: ...
    def transfer_set_speed_limits_mode(
        self,
        intended_state: Optional[bool] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def transfer_setSpeedLimitsMode(
        self,
        intended_state: Optional[bool] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def transfer_toggleSpeedLimitsMode(
        self,
        intended_state: Optional[bool] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def transfer_toggle_speed_limits_mode(
        self,
        intended_state: Optional[bool] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def transfer_download_limit(self, **kwargs: KwargsT) -> int: ...
    def transfer_downloadLimit(self, **kwargs: KwargsT) -> int: ...
    def transfer_upload_limit(self, **kwargs: KwargsT) -> int: ...
    def transfer_uploadLimit(self, **kwargs: KwargsT) -> int: ...
    def transfer_set_download_limit(
        self,
        limit: Optional[Text | int] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def transfer_setDownloadLimit(
        self,
        limit: Optional[Text | int] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def transfer_set_upload_limit(
        self,
        limit: Optional[Text | int] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def transfer_setUploadLimit(
        self,
        limit: Optional[Text | int] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def transfer_ban_peers(
        self,
        peers: Optional[Text | Iterable[Text]] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    def transfer_banPeers(
        self,
        peers: Optional[Text | Iterable[Text]] = None,
        **kwargs: KwargsT,
    ) -> None: ...



from enum import Enum
from typing import Any
from typing import Optional
from typing import Text
from typing import Type
from typing import TypeVar

try:
    from collections import UserList
except ImportError:
    from UserList import UserList  # type: ignore

from qbittorrentapi._attrdict import AttrDict
from qbittorrentapi._types import DictInputT
from qbittorrentapi._types import JsonDictionaryT
from qbittorrentapi._types import KwargsT
from qbittorrentapi._types import ListInputT
from qbittorrentapi.client import Client
from qbittorrentapi.request import Request

K = TypeVar("K")
V = TypeVar("V")
ListEntryT = TypeVar("ListEntryT", bound=JsonDictionaryT)

class APINames(Enum):
    Authorization: Text
    Application: Text
    Log: Text
    Sync: Text
    Transfer: Text
    Torrents: Text
    RSS: Text
    Search: Text
    EMPTY: Text

class TorrentStates(Enum):
    ERROR: Text
    MISSING_FILES: Text
    UPLOADING: Text
    PAUSED_UPLOAD: Text
    QUEUED_UPLOAD: Text
    STALLED_UPLOAD: Text
    CHECKING_UPLOAD: Text
    FORCED_UPLOAD: Text
    ALLOCATING: Text
    DOWNLOADING: Text
    METADATA_DOWNLOAD: Text
    FORCED_METADATA_DOWNLOAD: Text
    PAUSED_DOWNLOAD: Text
    QUEUED_DOWNLOAD: Text
    FORCED_DOWNLOAD: Text
    STALLED_DOWNLOAD: Text
    CHECKING_DOWNLOAD: Text
    CHECKING_RESUME_DATA: Text
    MOVING: Text
    UNKNOWN: Text
    @property
    def is_downloading(self) -> bool: ...
    @property
    def is_uploading(self) -> bool: ...
    @property
    def is_complete(self) -> bool: ...
    @property
    def is_checking(self) -> bool: ...
    @property
    def is_errored(self) -> bool: ...
    @property
    def is_paused(self) -> bool: ...

class ClientCache:
    _client: Client
    def __init__(self, *args: Any, client: Request, **kwargs: KwargsT) -> None: ...

class Dictionary(ClientCache, AttrDict[K, V]):
    def __init__(
        self,
        data: Optional[DictInputT] = None,
        client: Optional[Request] = None,
    ): ...
    @classmethod
    def _normalize(cls, data: DictInputT) -> AttrDict[K, V]: ...

class List(ClientCache, UserList[ListEntryT]):
    def __init__(
        self,
        list_entries: Optional[ListInputT] = None,
        entry_class: Optional[Type[ListEntryT]] = None,
        client: Optional[Request] = None,
    ) -> None: ...

class ListEntry(JsonDictionaryT): ...



from logging import Logger
from typing import Callable
from typing import Set
from typing import Text
from typing import Type
from typing import TypeVar

from qbittorrentapi.request import Request

logger: Logger

APIClassT = TypeVar("APIClassT", bound=Request)
APIReturnValueT = TypeVar("APIReturnValueT")

class alias:
    aliases: Set[Text]
    def __init__(self, *aliases: Text) -> None: ...
    def __call__(
        self, func: Callable[..., APIReturnValueT]
    ) -> Callable[..., APIReturnValueT]: ...

def aliased(aliased_class: Type[APIClassT]) -> Type[APIClassT]: ...
def login_required(
    func: Callable[..., APIReturnValueT]
) -> Callable[..., APIReturnValueT]: ...
def handle_hashes(
    func: Callable[..., APIReturnValueT]
) -> Callable[..., APIReturnValueT]: ...
def endpoint_introduced(
    version_introduced: Text,
    endpoint: Text,
) -> Callable[[Callable[..., APIReturnValueT]], Callable[..., APIReturnValueT]]: ...
def version_removed(
    version_obsoleted: Text,
    endpoint: Text,
) -> Callable[[Callable[..., APIReturnValueT]], Callable[..., APIReturnValueT]]: ...
def check_for_raise(client: Request, error_message: Text) -> None: ...



from logging import Logger
from typing import Any
from typing import Iterable
from typing import Mapping
from typing import MutableMapping
from typing import Optional
from typing import Text
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union
from urllib.parse import ParseResult

import six
from requests import Response
from requests import Session

from qbittorrentapi._types import FilesToSendT
from qbittorrentapi._types import JsonDictionaryT
from qbittorrentapi._types import KwargsT
from qbittorrentapi.app import Application
from qbittorrentapi.auth import Authorization
from qbittorrentapi.definitions import APINames
from qbittorrentapi.definitions import List
from qbittorrentapi.log import Log
from qbittorrentapi.rss import RSS
from qbittorrentapi.search import Search
from qbittorrentapi.sync import Sync
from qbittorrentapi.torrents import TorrentCategories
from qbittorrentapi.torrents import Torrents
from qbittorrentapi.torrents import TorrentTags
from qbittorrentapi.transfer import Transfer

logger: Logger

FinalResponseT = TypeVar(
    "FinalResponseT",
    bound=Union[
        int,
        bytes,
        six.text_type,
        JsonDictionaryT,
        List[JsonDictionaryT],
    ],
)

class URL(object):
    client: Request
    def __init__(self, client: Request) -> None: ...
    def build_url(
        self,
        api_namespace: APINames | Text,
        api_method: Text,
        headers: Mapping[Text, Text],
        requests_kwargs: Mapping[Text, Any],
    ) -> str: ...
    def build_base_url(
        self,
        headers: Mapping[Text, Text],
        requests_kwargs: Optional[Mapping[Text, Any]] = None,
    ) -> str: ...
    def detect_scheme(
        self,
        base_url: ParseResult,
        default_scheme: Text,
        alt_scheme: Text,
        headers: Mapping[Text, Text],
        requests_kwargs: Mapping[Text, Any],
    ) -> str: ...
    def build_url_path(
        self, api_namespace: APINames | Text, api_method: Text
    ) -> str: ...

class Request(object):
    host: Text
    port: Text | int
    username: Text
    _password: Text
    _url: URL
    _http_session: Session | None
    _application: Application | None
    _authorization: Authorization | None
    _transfer: Transfer | None
    _torrents: Torrents | None
    _torrent_categories: TorrentCategories | None
    _torrent_tags: TorrentTags | None
    _log: Log | None
    _sync: Sync | None
    _rss: RSS | None
    _search: Search | None

    _API_BASE_URL: Text | None
    _API_BASE_PATH: Text | None

    _EXTRA_HEADERS: Mapping[Text, Text] | None
    _REQUESTS_ARGS: MutableMapping[Text, Any] | None
    _VERIFY_WEBUI_CERTIFICATE: bool | None
    _FORCE_SCHEME_FROM_HOST: bool | None
    _RAISE_UNIMPLEMENTEDERROR_FOR_UNIMPLEMENTED_API_ENDPOINTS: bool | None
    _RAISE_NOTIMPLEMENTEDERROR_FOR_UNIMPLEMENTED_API_ENDPOINTS: bool | None
    _RAISE_UNSUPPORTEDVERSIONERROR: bool | None
    _VERBOSE_RESPONSE_LOGGING: bool | None
    _PRINT_STACK_FOR_EACH_REQUEST: bool | None
    _SIMPLE_RESPONSES: bool | None
    _DISABLE_LOGGING_DEBUG_OUTPUT: bool | None
    _MOCK_WEB_API_VERSION: Text | None
    def __init__(
        self,
        host: Optional[Text] = "",
        port: Optional[Text | int] = None,
        username: Optional[Text] = None,
        password: Optional[Text] = None,
        **kwargs: KwargsT
    ) -> None: ...
    def _initialize_context(self) -> None: ...
    def _initialize_lesser(
        self,
        EXTRA_HEADERS: Optional[Mapping[Text, Text]] = None,
        REQUESTS_ARGS: Optional[Mapping[Text, Any]] = None,
        VERIFY_WEBUI_CERTIFICATE: bool = True,
        FORCE_SCHEME_FROM_HOST: bool = False,
        RAISE_UNIMPLEMENTEDERROR_FOR_UNIMPLEMENTED_API_ENDPOINTS: bool = False,
        RAISE_NOTIMPLEMENTEDERROR_FOR_UNIMPLEMENTED_API_ENDPOINTS: bool = False,
        RAISE_ERROR_FOR_UNSUPPORTED_QBITTORRENT_VERSIONS: bool = False,
        VERBOSE_RESPONSE_LOGGING: bool = False,
        PRINT_STACK_FOR_EACH_REQUEST: bool = False,
        SIMPLE_RESPONSES: bool = False,
        DISABLE_LOGGING_DEBUG_OUTPUT: bool = False,
        MOCK_WEB_API_VERSION: Optional[Text] = None,
    ) -> None: ...
    @classmethod
    def _list2string(
        cls,
        input_list: Optional[Iterable[Any]] = None,
        delimiter: Text = "|",
    ) -> Text: ...
    def _trigger_session_initialization(self) -> None: ...
    def _get(
        self,
        _name: APINames | Text = APINames.EMPTY,
        _method: Text = "",
        requests_args: Optional[Mapping[Text, Any]] = None,
        requests_params: Optional[Mapping[Text, Any]] = None,
        headers: Optional[Mapping[Text, Text]] = None,
        params: Optional[Mapping[Text, Any]] = None,
        data: Optional[Mapping[Text, Any]] = None,
        files: Optional[FilesToSendT] = None,
        response_class: Optional[Type[FinalResponseT]] = None,
        **kwargs: KwargsT
    ) -> FinalResponseT: ...
    def _post(
        self,
        _name: APINames | Text = APINames.EMPTY,
        _method: Text = "",
        requests_args: Optional[Mapping[Text, Any]] = None,
        requests_params: Optional[Mapping[Text, Any]] = None,
        headers: Optional[Mapping[Text, Text]] = None,
        params: Optional[Mapping[Text, Any]] = None,
        data: Optional[Mapping[Text, Any]] = None,
        files: Optional[FilesToSendT] = None,
        response_class: Optional[Type[FinalResponseT]] = None,
        **kwargs: KwargsT
    ) -> FinalResponseT: ...
    def _request_manager(
        self,
        http_method: Text,
        api_namespace: APINames | Text,
        api_method: Text,
        _retries: int = 1,
        _retry_backoff_factor: float = 0.3,
        requests_args: Optional[Mapping[Text, Any]] = None,
        requests_params: Optional[Mapping[Text, Any]] = None,
        headers: Optional[Mapping[Text, Text]] = None,
        params: Optional[Mapping[Text, Any]] = None,
        data: Optional[Mapping[Text, Any]] = None,
        files: Optional[FilesToSendT] = None,
        response_class: Optional[Type[FinalResponseT]] = None,
        **kwargs: KwargsT
    ) -> FinalResponseT: ...
    def _request(
        self,
        http_method: Text,
        api_namespace: APINames | Text,
        api_method: Text,
        requests_args: Optional[Mapping[Text, Any]] = None,
        requests_params: Optional[Mapping[Text, Any]] = None,
        headers: Optional[Mapping[Text, Text]] = None,
        params: Optional[Mapping[Text, Any]] = None,
        data: Optional[Mapping[Text, Any]] = None,
        files: Optional[FilesToSendT] = None,
        response_class: Optional[Type[FinalResponseT]] = None,
        **kwargs: KwargsT
    ) -> FinalResponseT: ...
    @staticmethod
    def _get_response_kwargs(
        kwargs: MutableMapping[Text, Any]
    ) -> Tuple[dict[str, Any], dict[str, Any]]: ...
    def _get_requests_kwargs(
        self,
        requests_args: Optional[Mapping[Text, Any]] = None,
        requests_params: Optional[Mapping[Text, Any]] = None,
    ) -> dict[Text, Any]: ...
    @staticmethod
    def _get_headers(
        headers: Optional[Mapping[Text, Text]] = None,
        more_headers: Optional[Mapping[Text, Text]] = None,
    ) -> dict[Text, Text]: ...
    @staticmethod
    def _get_data(
        http_method: Text,
        params: Optional[Mapping[Text, Any]] = None,
        data: Optional[Mapping[Text, Any]] = None,
        files: Optional[FilesToSendT] = None,
        **kwargs: KwargsT
    ) -> Tuple[dict[Text, Any], dict[Text, Any], FilesToSendT]: ...
    def _cast(
        self,
        response: Response,
        response_class: Type[FinalResponseT],
        **response_kwargs: KwargsT
    ) -> FinalResponseT: ...
    @property
    def _session(self) -> Session: ...
    @staticmethod
    def _handle_error_responses(
        data: Mapping[Text, Any],
        params: Mapping[Text, Any],
        response: Response,
    ) -> None: ...
    def _verbose_logging(
        self,
        http_method: Text,
        url: Text,
        data: Mapping[Text, Any],
        params: Mapping[Text, Any],
        requests_kwargs: Mapping[Text, Any],
        response: Response,
    ) -> None: ...



from typing import Mapping
from typing import Optional
from typing import Text

from qbittorrentapi._types import JsonDictionaryT
from qbittorrentapi._types import JsonValueT
from qbittorrentapi._types import KwargsT
from qbittorrentapi.app import AppAPIMixIn
from qbittorrentapi.definitions import ClientCache

class RSSitemsDictionary(JsonDictionaryT): ...
class RSSRulesDictionary(JsonDictionaryT): ...

class RSS(ClientCache):
    items: _Items
    def __init__(self, client: RSSAPIMixIn) -> None: ...
    def add_folder(
        self,
        folder_path: Optional[Text] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    addFolder = add_folder
    def add_feed(
        self,
        url: Optional[Text] = None,
        item_path: Optional[Text] = None,
        **kwargs: KwargsT
    ) -> None: ...
    addFeed = add_feed
    def remove_item(
        self,
        item_path: Optional[Text] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    removeItem = remove_item
    def move_item(
        self,
        orig_item_path: Optional[Text] = None,
        new_item_path: Optional[Text] = None,
        **kwargs: KwargsT
    ) -> None: ...
    moveItem = move_item
    def refresh_item(self, item_path: Optional[Text] = None) -> None: ...
    refreshItem = refresh_item
    def mark_as_read(
        self,
        item_path: Optional[Text] = None,
        article_id: Optional[Text | int] = None,
        **kwargs: KwargsT
    ) -> None: ...
    markAsRead = mark_as_read
    def set_rule(
        self,
        rule_name: Optional[Text] = None,
        rule_def: Optional[Mapping[Text, JsonValueT]] = None,
        **kwargs: KwargsT
    ) -> None: ...
    setRule = set_rule
    def rename_rule(
        self,
        orig_rule_name: Optional[Text] = None,
        new_rule_name: Optional[Text] = None,
        **kwargs: KwargsT
    ) -> None: ...
    renameRule = rename_rule
    def remove_rule(
        self,
        rule_name: Optional[Text] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    removeRule = remove_rule
    @property
    def rules(self) -> RSSRulesDictionary: ...
    def matching_articles(
        self,
        rule_name: Optional[Text] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    matchingArticles = matching_articles

    class _Items(ClientCache):
        def __call__(
            self,
            include_feed_data: Optional[bool] = None,
            **kwargs: KwargsT,
        ) -> RSSitemsDictionary: ...
        @property
        def without_data(self) -> RSSitemsDictionary: ...
        @property
        def with_data(self) -> RSSitemsDictionary: ...

class RSSAPIMixIn(AppAPIMixIn):
    @property
    def rss(self) -> RSS: ...
    def rss_add_folder(
        self,
        folder_path: Optional[Text] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    rss_addFolder = rss_add_folder
    def rss_add_feed(
        self,
        url: Optional[Text] = None,
        item_path: Optional[Text] = None,
        **kwargs: KwargsT
    ) -> None: ...
    rss_addFeed = rss_add_feed
    def rss_remove_item(
        self,
        item_path: Optional[Text] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    rss_removeItem = rss_remove_item
    def rss_move_item(
        self,
        orig_item_path: Optional[Text] = None,
        new_item_path: Optional[Text] = None,
        **kwargs: KwargsT
    ) -> None: ...
    rss_moveItem = rss_move_item
    def rss_items(
        self,
        include_feed_data: Optional[bool] = None,
        **kwargs: KwargsT,
    ) -> RSSitemsDictionary: ...
    def rss_refresh_item(
        self,
        item_path: Optional[Text] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    rss_refreshItem = rss_refresh_item
    def rss_mark_as_read(
        self,
        item_path: Optional[Text] = None,
        article_id: Optional[Text | int] = None,
        **kwargs: KwargsT
    ) -> None: ...
    rss_markAsRead = rss_mark_as_read
    def rss_set_rule(
        self,
        rule_name: Optional[Text] = None,
        rule_def: Optional[Mapping[Text, JsonValueT]] = None,
        **kwargs: KwargsT
    ) -> None: ...
    rss_setRule = rss_set_rule
    def rss_rename_rule(
        self,
        orig_rule_name: Optional[Text] = None,
        new_rule_name: Optional[Text] = None,
        **kwargs: KwargsT
    ) -> None: ...
    rss_renameRule = rss_rename_rule
    def rss_remove_rule(
        self,
        rule_name: Optional[Text] = None,
        **kwargs: KwargsT,
    ) -> None: ...
    rss_removeRule = rss_remove_rule
    def rss_rules(self, **kwargs: KwargsT) -> RSSRulesDictionary: ...
    def rss_matching_articles(
        self,
        rule_name: Optional[Text] = None,
        **kwargs: KwargsT,
    ) -> RSSitemsDictionary: ...
    rss_matchingArticles = rss_matching_articles



from typing import IO
from typing import Any
from typing import Iterable
from typing import Mapping
from typing import Sequence
from typing import Text
from typing import Tuple
from typing import Union

from qbittorrentapi.definitions import Dictionary

KwargsT = Any
# Type to define JSON
JsonValueT = Union[
    None,
    int,
    Text,
    bool,
    Sequence[JsonValueT],
    Mapping[Text, JsonValueT],
]
JsonDictionaryT = Dictionary[Text, JsonValueT]
# Type for inputs to build a Dictionary
DictInputT = Mapping[Text, JsonValueT]
# Type for inputs to build a List
ListInputT = Iterable[Mapping[Text, JsonValueT]]
# Type for `files` in requests.get()/post()
FilesToSendT = Mapping[Text, IO[bytes] | Tuple[Text, IO[bytes]]]
